'''
Created on Oct 5, 2009

@author: kenji
'''
import gst
import core.VideoElements
import gobject

########################### AUXILIARY FUNCTIONS ###########################

def __move_n_frames(pipeline, n=1):
    for i in range(0, pipeline.num_videos):
        element = pipeline.find_element("identity_" + str(pipeline.xid_list[i]))
        current_frame = core.VideoElements.query_position(element)
        n -= 1
        core.VideoElements.seek_frame(current_frame + n, element)
    
def __move(pipeline, offset=1):
    '''Change the pipeline frame from current frame plus the offset
    @param button: button
    @param pipeline: pipeline
    @param offset: offset'''
    # Retrieves the pipeline's state
    states = pipeline.pipeline.get_state()
    for i in range(0, len(states)):
        if isinstance(states[i], gst.State):
            if states[i] == gst.STATE_PLAYING:
                current_state = gst.STATE_PLAYING
            elif states[i] == gst.STATE_PAUSED:
                current_state = gst.STATE_PAUSED
    # Pauses the pipeline so that we can manipulate the videos without losing sync
    pipeline.pipeline.set_state(gst.STATE_PAUSED)
    # Manipulating the videos
    min = offset
    for i in range(0, pipeline.num_videos):
        element = pipeline.find_element("identity_" + pipeline.xid_list[i].__str__())
        current_frame = core.VideoElements.query_position(element)
        total_frame = core.VideoElements.query_duration(element)
        if current_frame + min < 0:
            min = -current_frame
        elif current_frame + min > total_frame:
            min = total_frame - current_frame
    __move_n_frames(pipeline, min)
    # Returns the pipeline to its original state
    pipeline.pipeline.set_state(current_state)

##################### VideoElements Callback ##############################

def cb_decoder(decoder, pad, data, element_sink_pad):
    '''
    Links decoder to the next element using proper pads
    @param decoder: decoder
    @param pad: pad
    @param data: data
    @param element: next element
    '''
    structureName = pad.get_caps()[0].get_name()
    if structureName.startswith("video"):
#        sink = element.get_pad("sink")
        if not element_sink_pad.is_linked():
            pad.link(element_sink_pad)

##################### VideoPipeline Callback ##############################
def cb_update_label(pipeline, xid_list, widgets_callback_dictionary):
    '''
    Updates the label position
    @param label: label
    @param pipeline: pipeline
    @param xid_list: xid_list
    '''
    for i in range(0, len(xid_list)):
        label = widgets_callback_dictionary['Label_Current_Frame_' + str(i)]
        label_position = core.VideoElements.query_position(pipeline.get_by_name("identity_" + str(xid_list[i])))
        if int(label.get_text()) != label_position:
            label.set_text(str(label_position))
    return True
                
def cb_update_trackbar(trackbar, pipeline, xid_list, widgets_callback_dictionary):
    '''
    Updates the trackbar position
    @param trackbar: trackbar
    @param pipeline: pipeline
    @param xid_list: xid_list
    '''
    for i in range(0, len(xid_list)):
        trackbar = widgets_callback_dictionary['Callback_Trackbar_' + i.__str__()]
        video_position = core.VideoElements.query_position(pipeline.get_by_name("identity_" + str(xid_list[i])))
        if trackbar.get_value() != video_position:
            trackbar.set_value(float(video_position))
    return True

def cb_play(button, pipeline):
    '''
    Change state of the pipeline to playing
    @param button: button
    '''
    pipeline.set_state(gst.STATE_PLAYING)

def cb_pause(button, pipeline):
    '''
    Change state of the pipeline to paused
    @param button: button
    '''
    pipeline.set_state(gst.STATE_PAUSED)
        
def cb_prev(button, pipeline):
    '''Goes back to the first position of each video
    @param button: gtk.Button
    @param pipeline: gst.Pipeline'''
    # Pauses the pipeline
    pipeline.pipeline.set_state(gst.STATE_PAUSED)
    # Manipulating the videos
    element = pipeline.find_element("identity_" + pipeline.xid_list[0].__str__())
    current_frame = core.VideoElements.query_position(element)
    min = current_frame
    for i in range(1, pipeline.num_videos):
        element = pipeline.find_element("identity_" + pipeline.xid_list[i].__str__())
        current_frame = core.VideoElements.query_position(element)
        if current_frame < min:
            min = current_frame
    __move_n_frames(pipeline, -min + 1)
        
def cb_next(button, pipeline):
    '''Advances to the last position of each video
    @param button: gtk.Button
    @param pipeline: gst.Pipeline'''
    # Pauses the pipeline
    pipeline.pipeline.set_state(gst.STATE_PAUSED)
    # Manipulating the videos
    element = pipeline.find_element("identity_" + pipeline.xid_list[0].__str__())
    current_frame = core.VideoElements.query_position(element)
    total_frame = core.VideoElements.query_duration(element)
    min = total_frame - current_frame
    for i in range(0, pipeline.num_videos):
        element = pipeline.find_element("identity_" + pipeline.xid_list[i].__str__())
        current_frame = core.VideoElements.query_position(element)
        total_frame = core.VideoElements.query_duration(element)
        if total_frame - current_frame < min:
            min = total_frame - current_frame
    __move_n_frames(pipeline, min)

def cb_rewind(button, pipeline, widgets_callback_dictionary):
    value = widgets_callback_dictionary["Callback_Entry_Rewind"].get_text()
    if value.isdigit():
        __move(pipeline, -1 * long(value))

def cb_fastforward(button, pipeline, widgets_callback_dictionary):
    value = widgets_callback_dictionary["Callback_Entry_FastForward"].get_text()
    if value.isdigit():
        __move(pipeline, long(value))

def cb_test_seek(button, pipeline, widgets_callback_dictionary):
    value = widgets_callback_dictionary["Callback_Entry_Next"].get_text()
    if value.isdigit():
        for i in range(0, pipeline.num_videos):
            element = pipeline.find_element("identity_" + pipeline.xid_list[i].__str__())
            core.VideoElements.seek_frame(int(value), element)
