#!/usr/bin/env python
'''
Created on Sep 17, 2009

@author: kenji
'''
import pygst
pygst.require("0.10")
import gst
from callback.CoreCallbackLib import *

class VideoElements:
    '''
    Elements that constitute a video dataflow
    '''
    elements = {}
    
    def __init__(self, elements_list, xid):
        '''
        Creation of the basic elements of Pipeline
        @param elements_list: List of elements
        @param xid: xwindow's id
        '''
        self.elements = {}
        for i in range (0, elements_list.__len__()):
            try:
                #setting up the element name with the drawing area's xwindow id with its own name
                self.elements[elements_list[i]] = gst.element_factory_make(elements_list[i], elements_list[i] + "_" + str(xid))
            except gst.PluginNotFoundError:
                gst.error("Element %s could not be created" % elements_list[i])

def set_source_location(bin, location):
    '''
    Setting file source's location
    @param bin: Bin
    '''
    bin.elements['filesrc'].set_property("location", location)

#===============================================================================
# FIXME: Verificar o erro na recuperacao de um frame
#===============================================================================

def seek_frame(frame, element):
    '''
    Seeks for a specific frame
    @param time_format: time format
    @param frame: frame
    @param element: element
    @return: boolean
    '''
    time_format = gst.Format(gst.FORMAT_DEFAULT)
    result = element.seek_simple(time_format, gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE, frame)
    if not result:
        gst.info(("Seeking to frame %d failed") % frame)
    return result

def query_position(element):
    '''
    Current position in the elements stream
    @param time_format: time format
    @param element: element
    @return: 
    '''
    time = gst.Format(gst.FORMAT_TIME)
    time_format = gst.Format(gst.FORMAT_DEFAULT)
    position_time = element.query_position(time, None)[0]
    result = element.query_convert(time, position_time, time_format)[1]
    return result

def query_duration(element):
    '''
    Total duration of element stream
    @param time_format: time format
    @param element: element
    @return: 
    '''
    time = gst.Format(gst.FORMAT_TIME)
    time_format = gst.Format(gst.FORMAT_DEFAULT)
    position_time = element.query_duration(time, None)[0]
    result = element.query_convert(time, position_time, time_format)[1]
#    time_format = gst.Format(gst.FORMAT_DEFAULT)
#    result = element.query_duration(time_format, None)[0]
    return result

def set_output_ui(video_out, xid):
    '''
    Binds the video output to a gtk's drawing area
    @param video_out: video_out
    @param xid: window.xid
    '''
    if video_out.get_property("force-aspect-ratio") != None:
        video_out.set_property("force-aspect-ratio", True)
    video_out.set_xwindow_id(xid)
