#!/usr/bin/env python
'''
Created on Sep 18, 2009

@author: kenji
'''
import pygst
pygst.require("0.10")
import gst
import VideoElements
import gobject

from callback.CoreCallbackLib import *


class VideoPipeline:
    '''
    Main Pipeline
    '''
    pipeline = None
    num_videos = 0
    xid_list = []
    widgets_callback_dict = []
    update_trackbar_cb_id = []
    update_label_cb_id = []
    frame_test = 0
    player_bus = None
    message_src = None
    
    multiqueue_element = None

    def __init__(self, num_videos=0, xid_list=None, widgets_callback_dict=None, player_bus=player_bus):
        '''
        Setting the pipeline up
        @param name: Name of pipeline
        '''
        self.message_src = gst.element_factory_make("identity", "dummy_element")

        self.player_bus = player_bus
        self.pipeline = gst.Pipeline()
        self.num_videos = num_videos
        self.xid_list = xid_list
        self.widgets_callback_dict = widgets_callback_dict
        self.multiqueue_element = gst.element_factory_make("multiqueue")
        
        #setting up multiqueue properties to optimize sincronization
        self.multiqueue_element.set_property("max-size-bytes", 1000000)
        self.multiqueue_element.set_property("max-size-time", 14000000)
        
        self.pipeline.add(self.multiqueue_element)

    def setup_pipeline(self, source_locations_list):
        '''
        Setting pipeline up
        @param source_locations_list: List of the source locations
        '''
        video_elements = []
        video_elements.append('filesrc')
        video_elements.append('decodebin2')
        video_elements.append('identity')
        video_elements.append('xvimagesink')
        #=======================================================================
        # TODO: Tentar mudar o xvimagesink para autovideosink
        #=======================================================================

        for i in range (0, self.num_videos):
            bin = VideoElements.VideoElements(video_elements, self.xid_list[i])
            
            #adding elements into pipeline
            self.pipeline_add(bin)
            
            #setting file location
            VideoElements.set_source_location(bin, source_locations_list[i])

            #getting multiqueue's src and sink pads
            sink_pad = self.multiqueue_element.get_request_pad("sink%d")
            src_pad = self.multiqueue_element.get_pad("src" + i.__str__())
            
            #setting callbacks up
            bin.elements['decodebin2'].connect("new-decoded-pad", VideoElements.cb_decoder, sink_pad)
                    
            #linking elements
            gst.element_link_many(bin.elements['filesrc'], bin.elements['decodebin2'])
            self.multiqueue_element.link_pads(src_pad, bin.elements['identity'], bin.elements['identity'].get_pad("sink"))
            gst.element_link_many(bin.elements['identity'], bin.elements['xvimagesink'])

        #setting pipeline's bus
        bus = self.pipeline.get_bus()
        bus.add_signal_watch()
        bus.enable_sync_message_emission()
        bus.connect("message", self.cb_on_message)
        bus.connect("sync-message::element", self.cb_on_sync_message)
        #
        self.pipeline.set_state(gst.STATE_PAUSED)
    
    def pipeline_add(self, bin):
        '''
        Adding elements of bin into pipeline
        @param bin: bin with elements
        '''
        for key in bin.elements.keys():
            self.pipeline.add(bin.elements[key])
    
    def find_element(self, name):
        '''
        Finds some element inside the pipeline
        @param name: element's name
        @return: element
        '''
        iterator = self.pipeline.elements()
        while True:
            try:
                element = iterator.next()
                if element.get_name() == name:
                    return element
            except StopIteration:
                return None
    
    def cb_on_message(self, bus, message):
        '''
        Receives a message from the bus on a 'message' signal
        @param bus: bus
        @param message: message
        '''
        type = message.type
        if type == gst.MESSAGE_EOS:
            self.pipeline.set_state(gst.STATE_PLAYING)
        elif type == gst.MESSAGE_ERROR:
            err, debug = message.parse_error()
            print "Error: %s" % err, debug
        elif type == gst.MESSAGE_WARNING:
            warn, debug = message.parse_warning()
            print "Warning: %s" % warn, debug
        elif type == gst.MESSAGE_STATE_CHANGED:
            parsed = message.parse_state_changed()
            if message.src.get_name().startswith('pipeline') and parsed[0] == gst.STATE_READY and parsed[1] == gst.STATE_PAUSED:
                message = gst.message_new_custom(gst.MESSAGE_ANY, self.message_src, gst.Structure("prepare-ui"))
                self.player_bus.post(message)
                

    def cb_on_sync_message(self, bus, message):
        #=======================================================================
        #TODO: Metodo que vai precisar chamar um popup com erro sobre codec
        #=======================================================================
        '''
        Receives a message from the bus on a 'sync-message::element' signal
        @param bus: bus
        @param message: message
        '''
        if message.structure is None:
            return
        messageName = message.structure.get_name()
        if messageName == "missing-plugin":
            print "Missing plugin:", message.structure['name']
        elif messageName == "prepare-xwindow-id":
            #binding the video output to the window
            imagesink = message.src
            #element's name includes the xwindow id (e.g. "xvimagesink_56783")
            xid = imagesink.get_name().split("_")[1]
            VideoElements.set_output_ui(imagesink, int(xid))
            
    def cb_pipeline_cleaning_up(self, data, event):
        '''Cleaning up the pipeline before the pipeline's window closes
        @param data: data
        @param event: event'''
        #stops updating the trackbar's position
        for i in range(0, self.num_videos):
            gobject.source_remove(self.update_trackbar_cb_id[i])
            gobject.source_remove(self.update_label_cb_id[i])
        self.pipeline.set_state(gst.STATE_NULL)
        gobject.source_remove(self.frame_test)
        
