'''
Created on Sep 20, 2009

@author: kenji
'''
from core.VideoPipeline import VideoPipeline
import sys
sys.path.append("./view")
sys.path.append("./core")

from view import *
import core.VideoPipeline
import core.VideoElements
import gst
import gobject

from callback.ViewCallbackLib import *
from callback.CoreCallbackLib import *

class Player():
    drawing_areas_list = []
    widgets_callback_dictionary = {}
    drawing_areas_xid_list = []
    source_location_list = []
    window = None
    video = None
    bus = None
    
    def __init__(self, main_window_callback_dictionary, source_location_list, initial_frame_list):
        self.drawing_areas_list = []
        self.widgets_callback_dictionary = {}
        self.drawing_areas_xid_list = []
        self.window = None
        self.video = None
        self.source_location_list = source_location_list
        self.initial_frame_list = initial_frame_list
        self.bus = gst.Bus()
        
        self.bus.set_sync_handler(self.message_handler)
        
        #getting the number of columns and lines set by the user
        lines, columns = self.__get_lines_and_columns(main_window_callback_dictionary)
        num_videos = int(columns) * int(lines)
        #setting the window properties
        self.__set_window_properties(lines, columns)
        #setting the video properties
        self.__set_video_properties(num_videos)
        #cleaning up the pipeline before the window closes
        self.window.connect("delete_event", self.video.cb_pipeline_cleaning_up)
        #setting the callbacks up
        self.__setup_callbacks(main_window_callback_dictionary) 
#        self.__set_video_initial_position(num_videos)
        
    def message_handler(self, bus, message):
        '''
        Message handler of the Player's bus
        @param bus: bus
        @param message: message
        '''
        #message to indicate that the trackbar and frame label can be initialized, this message will be sent by the pipeline
        if message.structure.get_name() == "prepare-ui":
            
            #initializes the trackbar after the pipeline becomes paused
            for i in range(0, self.video.num_videos):
                trackbar = self.widgets_callback_dictionary["Callback_Trackbar_" + i.__str__()]
                adjustment = trackbar.get_adjustment()
                for j in range (0, self.video.num_videos):
                    element = self.video.find_element("identity_" + self.video.xid_list[i].__str__())
                    if element != None:
                        break
                if element == None:
                    return
            
                upper = core.VideoElements.query_duration(element)
                adjustment.set_property('upper', upper)
                adjustment.set_property('step_increment', 1)
                adjustment.set_property('page_increment', 0.1 * upper)
                
                trackbar.set_digits(0)
            
                #===============================================================
                # TODO: Posteriormente encontrar o framerate do video e dividir 1 segundo pelo framerate
                #===============================================================
                
                # calls the cb_update_trackbar every 1000/30 milliseconds
                self.video.update_trackbar_cb_id.append(gobject.timeout_add(1000/30, cb_update_trackbar, trackbar, self.video.pipeline, self.video.xid_list, self.widgets_callback_dictionary))

                #initializes the labels
                total_frame = self.widgets_callback_dictionary['Label_Total_Frame_' + i.__str__()]
                num_total_frames = core.VideoElements.query_duration(self.video.find_element("identity_" + self.video.xid_list[i].__str__()))
            
                total_frame.set_text(num_total_frames.__str__())
                
                video_name = self.widgets_callback_dictionary['Label_Video_Name_' + i.__str__()]
                
                video_name.set_text(self.video.find_element("filesrc_" + self.video.xid_list[i].__str__()).get_property("location"))

                current_frame = self.widgets_callback_dictionary['Label_Current_Frame_' + i.__str__()]

                # calls the cb_update_trackbar every 1000/10 milliseconds
                self.video.update_label_cb_id.append(gobject.timeout_add(1000/10, cb_update_label, self.video.pipeline, self.video.xid_list, self.widgets_callback_dictionary))

#            self.video.frame_test = gobject.timeout_add(3000, self.verificacao_de_frame)
            self.__set_video_initial_position(self.video.num_videos)

        return gst.BUS_DROP

    def __get_lines_and_columns(self, main_window_callback_dictionary):
        '''
        Get the lines and columns set by the user
        @param main_window_callback_dictionary: main_window_callback_dictionary
        '''
        lines = int(main_window_callback_dictionary["Callback_Toolbar_X_Display_Control"].get_children()[0].get_children()[1].get_value())
        columns = int(main_window_callback_dictionary["Callback_Toolbar_Y_Display_Control"].get_children()[0].get_children()[1].get_value())
        return lines, columns
    
    def __set_window_properties(self, lines, columns):
        '''
        Set the view window properties
        @param lines: lines
        @param columns: columns
        '''
        #creating the player's window
        self.window = WindowFactory.create_view_window(self.widgets_callback_dictionary, self.drawing_areas_list, lines, columns)
        self.window.show_all()
        #feeding the drawing areas xid's list 
        for i in range(0, len(self.drawing_areas_list)):
            self.drawing_areas_xid_list.append(self.drawing_areas_list[i].window.xid)
        self.drawing_areas_list[0].get_parent().modify_bg(gtk.STATE_NORMAL, self.drawing_areas_list[i].get_parent().get_colormap().alloc_color("green"))
    
    def __set_video_properties(self, num_videos):
        '''
        Set the pipeline's properties
        @param num_videos: num_videos
        '''
        #creating video's pipeline
        self.video = core.VideoPipeline.VideoPipeline(num_videos=num_videos, xid_list=self.drawing_areas_xid_list, widgets_callback_dict=self.widgets_callback_dictionary, player_bus=self.bus)
        #setting up pipeline
        self.video.setup_pipeline(self.source_location_list)
    
    def __set_video_initial_position(self, num_videos):
        for i in range(num_videos):
            core.VideoElements.seek_frame(int(self.initial_frame_list[i]), self.video.find_element("identity_" + self.video.xid_list[i].__str__()))
    
    def __setup_callbacks(self, main_window_callback_dictionary):
        '''
        Setup window's callbacks
        @param main_window_callback_dictionary: main_window_callback_dictionary
        '''
        # Play Button and Pause Button
        play_button = self.widgets_callback_dictionary["Callback_Button_Play"]
        pause_button = self.widgets_callback_dictionary["Callback_Button_Pause"]
        play_button.connect("clicked", cb_click_play_button, pause_button)
        pause_button.connect("clicked", cb_click_pause_button, play_button)
        play_button.connect("clicked", cb_play, self.video.pipeline)
        pause_button.connect("clicked", cb_pause, self.video.pipeline)
        play_button.set_relief(gtk.RELIEF_NONE)
        pause_button.set_relief(gtk.RELIEF_NONE)

        # FF Button
        fastforward_button = self.widgets_callback_dictionary["Callback_Button_Fastforward"]
        fastforward_button.connect("clicked", cb_fastforward, self.video, self.widgets_callback_dictionary)
        fastforward_button.set_relief(gtk.RELIEF_NONE)

        # RW Button
        rewind_button = self.widgets_callback_dictionary["Callback_Button_Rewind"]
        rewind_button.connect("clicked", cb_rewind, self.video, self.widgets_callback_dictionary)
        rewind_button.set_relief(gtk.RELIEF_NONE)
        
        # Prev Button
        prev_button = self.widgets_callback_dictionary["Callback_Button_Previous"]
        prev_button.connect("clicked", cb_click_stop_button, play_button, pause_button)
        prev_button.connect("clicked", cb_prev, self.video)
        prev_button.set_relief(gtk.RELIEF_NONE)
        
        # Next Button
        next_button = self.widgets_callback_dictionary["Callback_Button_Next"]
        next_button.connect("clicked", cb_click_stop_button, play_button, pause_button)
        next_button.connect("clicked", cb_next, self.video)
        next_button.set_relief(gtk.RELIEF_NONE)
#        next_button = self.widgets_callback_dictionary["Callback_Button_Next"]
#        next_button.connect("clicked", cb_click_stop_button, play_button, pause_button)
#        next_button.connect("clicked", cb_test_seek, self.video, self.widgets_callback_dictionary)
#        next_button.set_relief(gtk.RELIEF_NONE)
        
        # Trackbar
        for i in range(0, self.video.num_videos):
            trackbar = self.widgets_callback_dictionary["Callback_Trackbar_" + i.__str__()]
            trackbar.connect("change-value", cb_change_trackbar_value, self.video)
            
        for i in range(0, self.video.num_videos):
            self.drawing_areas_list[i].add_events(gtk.gdk.BUTTON_PRESS_MASK)
            self.drawing_areas_list[i].connect("button_press_event", self.selection_manager)
        
    def selection_manager(self, widget, event):
        '''
        Manager that manages the selection of a drawing area and it's consequences
        @param widget: widget
        @param event: event
        '''
        selected_xid = widget.window.xid
        trackbar = self.widgets_callback_dictionary['Panel_Trackbar_' + self.video.xid_list.index(selected_xid).__str__()]
        statebar = self.widgets_callback_dictionary['Panel_Statebar_' + self.video.xid_list.index(selected_xid).__str__()]
        # Hiding all structures
        for i in range(0, self.video.num_videos):
            if selected_xid != self.video.xid_list[i]:
                not_visible_trackbar = self.widgets_callback_dictionary['Panel_Trackbar_' + self.video.xid_list.index(self.video.xid_list[i]).__str__()]
                not_visible_trackbar.hide()
                not_visible_statebar = self.widgets_callback_dictionary['Panel_Statebar_' + self.video.xid_list.index(self.video.xid_list[i]).__str__()]
                not_visible_statebar.hide()
                self.drawing_areas_list[i].get_parent().modify_bg(gtk.STATE_NORMAL, self.drawing_areas_list[i].get_parent().get_colormap().alloc_color("gray84"))
        # Showing the only structure selected
        trackbar.set_no_show_all(False)
        trackbar.show_all()
        statebar.set_no_show_all(False)
        statebar.show_all()
        # Highlight the selected video border
        widget.get_parent().modify_bg(gtk.STATE_NORMAL, self.drawing_areas_list[i].get_parent().get_colormap().alloc_color("green"))

#################################### BEGIN FRAME TEST ####################################
    def verificacao_de_frame(self):
        self.video.pipeline.set_state(gst.STATE_PAUSED)
        print "----------------- BEGIN FRAME -----------------"
        for i in range(0, self.video.num_videos):
            current_frame = core.VideoElements.query_position(self.video.find_element("identity_" + str(self.video.xid_list[i])))
            print "video: " + str(i) + " frame: " + str(current_frame)
        print "----------------- END FRAME -----------------"
        self.video.pipeline.set_state(gst.STATE_PLAYING)
        return True
#################################### END FRAME TEST ####################################

def __initial_frames_validator(main_window_callback_dictionary):
    '''Validate the value of all initial frame entries on the Main Window
    @param main_window_callback_dictionary: a dictionary on which we'll insert the items that need a callback
    @return: True if all the entries are numeric values, False otherwise'''
    is_correct = True
    for key in main_window_callback_dictionary:
        if key.startswith("Callback_Frame_Filechooser_"):
            initial_frame_entry = main_window_callback_dictionary[key].get_child().get_children()[1].get_children()[1].get_child()
            text = initial_frame_entry.get_text()
            if not str(text).isdigit():
                #===============================================================
                # TODO: fazer o highlight aparecer na hora do click
                #===============================================================
                is_correct = False
    return is_correct

def cb_click_view_button(button, main_window_callback_dictionary):
    '''Connect the default click "click" Callback to a View button'''
    if not __initial_frames_validator(main_window_callback_dictionary):
        return
    source_location_list = []
    initial_frame_list = []
    table = main_window_callback_dictionary['Callback_Panel_Filechooser'].get_children()[0]
    for i in range(len(table.get_children())):
        source_location_list.append(None)
        initial_frame_list.append(None)
    for child in table.get_children():
        index = int(child.get_name().split("Callback_Frame_Filechooser_")[1])
        #=======================================================================
        # FIXME: Desvincular as entries com a interface e tambem dos spinbuttons
        #=======================================================================
        entry_value = child.get_children()[0].get_children()[0].get_children()[1].get_text()
        source_location_list[index] = entry_value
        initial_frame_value = child.get_children()[0].get_children()[1].get_children()[1].get_children()[0].get_text()
        initial_frame_list[index] = initial_frame_value
    player = Player(main_window_callback_dictionary, source_location_list, initial_frame_list)

def cb_change_trackbar_value(trackbar, scroll, new_value, player):
    #===========================================================================
    # FIXME: Para quando tem varios videos, as vezes a trackbar nao respeita o intervalo das outras
    #===========================================================================
    position_list = []
    trackbar_interval = new_value - trackbar.get_value()
    difference = 0
    for i in range(0, player.num_videos):
        element = player.find_element("identity_" + player.xid_list[i].__str__())
        element_current_position = core.VideoElements.query_position(element)
        element_interval = element_current_position + trackbar_interval
        position_list.append(element_interval)
        # Verifying if we got out of range of any of the trackbars
        duration = int(core.VideoElements.query_duration(element))
        if element_interval < 1:
            difference = 1 - element_interval
        elif element_interval > duration:
            difference = duration - element_interval
    for i in range(0, player.num_videos):
        core.VideoElements.seek_frame(position_list[i] + difference, player.find_element("identity_" + player.xid_list[i].__str__()))

if __name__ == '__main__':
    #===========================================================================
    # setting window up
    #===========================================================================
    
    #window
    widgets_callback_dictionary = {}
    drawing_areas_list = []
    window = WindowFactory.create_main_window(widgets_callback_dictionary, drawing_areas_list)
    window.show_all()
    
    #===========================================================================
    # setting callbacks up
    #===========================================================================
    
    # Lock Toggle Button
    lock_togglebutton = widgets_callback_dictionary["Callback_Toolbar_ToggleButton_Lock"]
    lock_togglebutton.connect("toggled", cb_click_lock_togglebutton, widgets_callback_dictionary)
    
    # View Button
    view_button = widgets_callback_dictionary["Callback_Button_View"]
    view_button.connect("clicked", cb_click_view_button, widgets_callback_dictionary)

    #gtk main
    gtk.gdk.threads_init()
    gtk.main()
