'''
Created on Aug 30, 2009

@author: marumjr
'''

#!/usr/bin/env python

import pygtk
pygtk.require('2.0')
import gtk

class MenuFactory():
    '''Menu Factory class, from which we extract some standard menus'''
    
    # Some useful masks
    CONTROL_SHIFT_MASK = gtk.gdk.CONTROL_MASK ^ gtk.gdk.SHIFT_MASK
    CONTROL_ALT_MASK = gtk.gdk.CONTROL_MASK ^ gtk.gdk.MOD1_MASK
    SHIFT_ALT_MASK = gtk.gdk.SHIFT_MASK ^ gtk.gdk.MOD1_MASK
    
    @staticmethod
    def __temp_function(widget, args=None):
        '''Temp function'''
        print "Item pressed: " + str(args)
    
    @staticmethod
    def __create_separator_item():
        '''Create a separator
        @return: gtk.Separator'''
        separator = gtk.SeparatorMenuItem()
        return separator
    
    @staticmethod
    def __create_default_menu_item(name, function=None, function_args=None, accel_group=None, shortcut_key=None, shortcut_mask=None):
        '''Create a general menu item
        @param name: the name of the new item
        @param function: a function to be added to the mennu item
        @param funtion_args: function arguments
        @param accel_group: gtk.AccelGroup
        @param shortcut_key: a string containing 1 letter corresponding to the shortcut key
        @param shortcut_mask: a mask of bits representing the modifiers for the shortcut key
        @return: gtk.MenuItem'''
        menu_item = gtk.MenuItem(name)
        menu_item.set_name("Callback_Menu" + name)
        menu_item.connect("activate", function, function_args)
        if accel_group != None and shortcut_key != None and shortcut_mask != None:
            menu_item.add_accelerator("activate", accel_group, ord(shortcut_key), shortcut_mask, gtk.ACCEL_VISIBLE)
        return menu_item
    
    @staticmethod
    def __create_default_image_menu_item(name, stock_item, function=None, function_args=None, accel_group=None, shortcut_key=None, shortcut_mask=None):
        '''Create a general menu item
        @param name: the name of the new item
        @param stock_item: a gtk stock item
        @param function: a function to be added to the mennu item
        @param funtion_args: function arguments
        @param accel_group: gtk.AccelGroup
        @param shortcut_key: a string containing 1 letter corresponding to the shortcut key
        @param shortcut_mask: a mask of bits representing the modifiers for the shortcut key
        @return: gtk.MenuItem'''
        menu_item = gtk.ImageMenuItem(stock_item)
        menu_item.set_name("Callback_Menu" + name)
        menu_item.connect("activate", function, function_args)
        if accel_group != None and shortcut_key != None and shortcut_mask != None:
            menu_item.add_accelerator("activate", accel_group, ord(shortcut_key), shortcut_mask, gtk.ACCEL_VISIBLE)
        return menu_item
    
    @staticmethod
    def __create_open_menu_item(accel_group):
        '''Create the "Open" menu option
        @param accel_group: gtk.AccelGroup
        @return: gtk.MenuItem'''
        item_name = "_Open"
        shortcut_key = "O"
        shortcut_mask = gtk.gdk.CONTROL_MASK
        menu_item = MenuFactory.__create_default_menu_item(item_name, MenuFactory.__temp_function, item_name, accel_group, shortcut_key, shortcut_mask)
        return menu_item
    
    @staticmethod
    def __create_save_menu_item(accel_group):
        '''Create the "Save" menu option
        @param accel_group: gtk.AccelGroup
        @return: gtk.MenuItem'''
        item_name = "_Save"
        shortcut_key = "S"
        shortcut_mask = gtk.gdk.CONTROL_MASK
        menu_item = MenuFactory.__create_default_menu_item(item_name, MenuFactory.__temp_function, item_name, accel_group, shortcut_key, shortcut_mask)
        return menu_item
    
    @staticmethod
    def __create_quit_menu_item(accel_group):
        '''Create the "Quit" menu option
        @param accel_group: gtk.AccelGroup
        @return: gtk.ImageMenuItem'''
        item_name = "_Quit"
        stock_item = gtk.STOCK_QUIT
        shortcut_key = "Q"
        shortcut_mask = gtk.gdk.CONTROL_MASK
        menu_item = MenuFactory.__create_default_image_menu_item(item_name, stock_item, MenuFactory.__temp_function, item_name, accel_group, shortcut_key, shortcut_mask)
        return menu_item
    
    @staticmethod
    def create_file_menu(widgets_callback_dictionary, accel_group):
        '''Create a file menu
        @param accel_group: gtk.AccelGroup
        @return: gtk.Menu'''
        file_menu = gtk.MenuItem("_File")
        file_submenu = gtk.Menu()
        # File submenu items
        open_item = MenuFactory.__create_open_menu_item(accel_group)
        widgets_callback_dictionary[open_item.get_name()] = open_item
        file_submenu.append(open_item)
        save_item = MenuFactory.__create_save_menu_item(accel_group)
        widgets_callback_dictionary[save_item.get_name()] = save_item
        file_submenu.append(save_item)
        separator = MenuFactory.__create_separator_item()
        file_submenu.append(separator)
        exit_item = MenuFactory.__create_quit_menu_item(accel_group)
        widgets_callback_dictionary[exit_item.get_name()] = exit_item
        file_submenu.append(exit_item)
        file_submenu.set_size_request(200, -1)
        # Connect the submenu to the main File menu
        file_menu.set_submenu(file_submenu)
        return file_menu
    
    @staticmethod
    def __create_about_menu_item(accel_group):
        '''Create the "About" menu option
        @param accel_group: gtk.AccelGroup
        @return: gtk.ImageMenuItem'''
        item_name = "_About"
        stock_item = gtk.STOCK_ABOUT
        shortcut_key = None
        shortcut_mask = None
        menu_item = MenuFactory.__create_default_image_menu_item(item_name, stock_item, MenuFactory.__temp_function, item_name, accel_group, shortcut_key, shortcut_mask)
        return menu_item
    
    @staticmethod
    def create_help_menu(widgets_callback_dictionary, accel_group):
        '''Create a help menu
        @param accel_group: gtk.AccelGroup
        @return: gtk.Menu'''
        help_menu = gtk.MenuItem("_Help")
        help_submenu = gtk.Menu()
        # Help submenu items
        about_item = MenuFactory.__create_about_menu_item(accel_group)
        widgets_callback_dictionary[about_item.get_name()] = about_item
        help_submenu.append(about_item)
        help_submenu.set_size_request(200, -1)
        # Connect the submenu to the main Help menu
        help_menu.set_submenu(help_submenu)
        return help_menu
