'''
Created on Oct 1, 2009

@author: marumjr
'''

#!/usr/bin/env python

from PanelFactory import *
import os

class WindowFactory():
    '''Window Factory class, from which we extract some standard windows'''
    
    # Some useful parameters
    screen_width = gtk.gdk.screen_width()
    screen_height = gtk.gdk.screen_height()
    
    @staticmethod
    def __delete_event(widget, event, data=None):
        '''Callback that interrupt the program'''
        gtk.main_quit()
        return False
    
    @staticmethod
    def __create_main_window(name, width, heigth):
        '''Create a general window
        @param name: the name of the window'''
        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.set_position(gtk.WIN_POS_CENTER)
        window.set_title(name)
        window.set_default_size(width, heigth)
        window.set_icon_from_file(os.path.abspath("./../img/logo.png"))
        window.connect("delete_event", WindowFactory.__delete_event)
        return window
    
    @staticmethod
    def __create_default_window(name, width, heigth):
        '''Create a general window
        @param name: the name of the window'''
        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.set_position(gtk.WIN_POS_CENTER)
        window.set_title(name)
        window.set_default_size(width, heigth)
        window.set_icon_from_file(os.path.abspath("./../img/logo.png"))
        return window
    
    @staticmethod
    def create_main_window(widgets_callback_dictionary, drawing_areas_list):
        '''Create the main window, on which program properties are set
        @param widgets_callback_dictionary: a dictionary containing widgets to be connected to a callback
        @param drawing_areas_list: a list containing all the drawing areas 
        @return: gtk.Window'''
        WindowFactory.screen_width *= 0.5
        WindowFactory.screen_height *= 0.5
        window = WindowFactory.__create_main_window("Main Window", int(WindowFactory.screen_width), int(WindowFactory.screen_height))
        vbox = gtk.VBox(False)
        # Window properties
        accel_group = gtk.AccelGroup()
        window.add_accel_group(accel_group)
        # Menu Panel
        menu_panel = PanelFactory.create_menu_panel(widgets_callback_dictionary, accel_group)
        vbox.pack_start(menu_panel, False)
        # Toolbar Panel
        toolbar_panel = PanelFactory.create_toolbar_panel(widgets_callback_dictionary)
        vbox.pack_start(toolbar_panel, False)
        # Filechooser Panel
        file_insertion_panel = PanelFactory.create_filechooser_panel(widgets_callback_dictionary)
        file_insertion_panel.set_no_show_all(True)
        vbox.pack_start(file_insertion_panel, True)
        # Flux Control Panel 
        flux_control_panel = PanelFactory.create_flux_control_panel(widgets_callback_dictionary)
        flux_control_panel.set_no_show_all(True)
        vbox.pack_start(flux_control_panel, False)
        window.add(vbox)
        return window
        
    @staticmethod
    def create_view_window(widgets_callback_dictionary, drawing_areas_list, lines = 1, columns = 1):
        '''Create the view window, on which the video is played
        @param widgets_callback_dictionary: a dictionary containing widgets to be connected to a callback
        @param drawing_areas_list: a list containing all the drawing areas
        @return: gtk.Window'''
        window = WindowFactory.__create_default_window("View Window", int(WindowFactory.screen_width), int(WindowFactory.screen_height))
        vbox = gtk.VBox(False)
        # Window properties
        accel_group = gtk.AccelGroup()
        window.add_accel_group(accel_group)
        # Video Panel
        video_panel = PanelFactory.create_video_panel(drawing_areas_list, lines, columns)
        vbox.pack_start(video_panel, True)
        # Separator
        separator = gtk.HSeparator()
        vbox.pack_start(separator, False, False, 5)
        # Video Trackbar
        for i in range(0, lines*columns):
            trackbar_panel = PanelFactory.create_trackbar_panel(widgets_callback_dictionary, i)
            if i != 0:
                trackbar_panel.set_no_show_all(True)
            vbox.pack_start(trackbar_panel, False)
        # Video Control Panel
        button_panel = PanelFactory.create_button_panel(widgets_callback_dictionary)
        vbox.pack_start(button_panel, False, False, 5)
        # Separator
        separator = gtk.HSeparator()
        vbox.pack_start(separator, False, False, 1)
        # Statebar Panel
        for i in range(0, lines*columns):
            statebar_panel = PanelFactory.create_statebar_panel(widgets_callback_dictionary, i)
            if i != 0:
                statebar_panel.set_no_show_all(True)
            vbox.pack_start(statebar_panel, False, False, 0)
        window.add(vbox)
        return window
        
